// MainWindow.cpp: Implementierungsdatei
//
// Copyright by Andr Stein
// E-Mail: stonemaster@steinsoft.net, andre_stein@web.de
// http://www.steinsoft.net
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "OpenGLMfc.h"
#include "MainWindow.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/*float LightPos[] = { 0.0f, 5.0f,-4.0f, 1.0f};			// Light Position
float LightAmb[] = { 0.2f, 0.2f, 0.2f, 1.0f};			// Ambient Light Values
float LightDif[] = { 0.6f, 0.6f, 0.6f, 1.0f};			// Diffuse Light Values
float LightSpc[] = {-0.2f, -0.2f, -0.2f, 1.0f};			// Specular Light Value*/

const float WHOLE_SCENE_LIGHT[] = { 0.25f, 0.25f, 0.25f, 1.0f}/*{0.5,0.5f,0.5f,1.0f}*/;
const float DIFFUSE_LIGHT[]  = { 0.6f, 0.6f, 0.6f, 1.0f}/*{0.5f,0.5f,0.5f,1.0f}*/;
const float SPECULAR_LIGHT[] = {-.1f, -0.1f, -0.1f, 1.0f}/*{0.5f,0.50f,0.50f,1.0f}*/;
const float LIGHT_POSITION1[] = { 0.0f, -5.0f,0.0f, 1.0f}/*{0.0f,0.0f,-5.0f,1.0f}*/;
const float LIGHT_POSITION2[] = { 0.0f, 5.0f,0.0f, 1.0f};


const float SPOT_DIRECTION[] = {0.0f,0.0f,-1.0f}; 

const float MATERIAL[] = {1.0f,1.0f,1.0f,1.0f};


/////////////////////////////////////////////////////////////////////////////
// CMainWindow

CMainWindow::CMainWindow()
{
	clientDC = NULL;
	fullscreen = true;
} 

CMainWindow::~CMainWindow()
{
	if (clientDC)
	{
		delete clientDC;
	}
}


BEGIN_MESSAGE_MAP(CMainWindow, CWnd)
	//{{AFX_MSG_MAP(CMainWindow)
	ON_WM_SIZE()
	ON_WM_CLOSE()
	ON_WM_SYSCOMMAND()
	ON_WM_CREATE()
	ON_WM_KEYDOWN()
	ON_WM_SETFOCUS()
	ON_WM_KILLFOCUS()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// Behandlungsroutinen fr Nachrichten CMainWindow 

void CMainWindow::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);
	
	if (cy == 0)								
	{
		cy = 1;						
	}

	glViewport(0, 0, cx, cy);	

	glMatrixMode(GL_PROJECTION);						
	glLoadIdentity();						

	
	gluPerspective(45.0f,(GLfloat)cx/(GLfloat)cy,0.1f,100.0f);

	glMatrixMode(GL_MODELVIEW);						
	glLoadIdentity();
}

void CMainWindow::Update()
{
	//Frametime is calculated
	//(for FPS independent graphics)
	static float framestart = timer.getTime();
	static float frameend = 0.0f;
	

	do 
	{
		frameend = timer.getTime();
	}
	while (frameend == framestart);

	frametime  = frameend - framestart;
	framestart = frameend;

	//*****************************************
	//                 MOUSE

	//Calculate mouse movement
	static CPoint lastMouse;

	GetCursorPos(&lastMouse);
	SetCursorPos(320,240);
		
	float moveX = float((320 - lastMouse.x))/100.0f;
	float moveY = float((240 - lastMouse.y))/100.0f;

	//Move spot according to the above mov.
	spot.move(-moveX,moveY);
	
	//*****************************************

	

	if (GetAsyncKeyState(VK_ESCAPE))
	{
		PostMessage(WM_CLOSE);
	}

	//Move spot up
	if (GetAsyncKeyState(VK_UP))
	{
		spot.move(0.0f,0.005f*frametime);
	}

	//Move down
	if (GetAsyncKeyState(VK_DOWN))
	{
		spot.move(0.0f,-0.005f*frametime);
	}

	if (GetAsyncKeyState(VK_LEFT))
	{
		spot.move(-0.005f*frametime,0.0f);
	}

	//Move down
	if (GetAsyncKeyState(VK_RIGHT))
	{
		spot.move(0.005f*frametime,0.0f);
	}

	DrawGLScene();
}

void CMainWindow::DrawGLScene()
{
	glClear(GL_COLOR_BUFFER_BIT |
			GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();

//	glTranslatef(7.5f,7.5f,0.0f);


	////////////////////
	// Drawing code here
	////////////////////

	room.draw();
	objects.draw();

	//************************
	// Projective Texture Mapping begins here
	
	spot.beginDraw(frametime);
	
	room.draw(true);
	objects.draw(true);
	
	spot.endDraw();

	// ...and ends here;)
	//*************************
		
	SwapBuffers(clientDC->m_hDC);
}

void CMainWindow::OnClose() 
{
	screenMode.reset();
	
	PostQuitMessage(0);

	ShowCursor(true);

	CWnd::OnClose();
}

void CMainWindow::InitGL()
{
	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClearDepth(1.0f);							
	glEnable(GL_DEPTH_TEST);					
	glDepthFunc(GL_LEQUAL);	
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glBlendFunc(GL_ONE,GL_ONE);

	/************************
			LIGHTING
	*************************/

	glLightModelfv(GL_LIGHT_MODEL_AMBIENT,WHOLE_SCENE_LIGHT);

	//Main
	glLightfv(GL_LIGHT0,GL_AMBIENT,SPECULAR_LIGHT);
	glLightfv(GL_LIGHT0,GL_SPECULAR,SPECULAR_LIGHT);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,DIFFUSE_LIGHT);
	glLightfv(GL_LIGHT0,GL_POSITION,LIGHT_POSITION1);

	glLightfv(GL_LIGHT1,GL_AMBIENT,SPECULAR_LIGHT);
	glLightfv(GL_LIGHT1,GL_SPECULAR,SPECULAR_LIGHT);
	glLightfv(GL_LIGHT1,GL_DIFFUSE,DIFFUSE_LIGHT);
	glLightfv(GL_LIGHT1,GL_POSITION,LIGHT_POSITION2);

	//Spotlight
	glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, 10.0f); //60 Spotlight
	glLightf(GL_LIGHT0,GL_SPOT_EXPONENT, 60.0f);
	glLightfv(GL_LIGHT0,GL_SPOT_DIRECTION,SPOT_DIRECTION);


	glColorMaterial(GL_FRONT,GL_AMBIENT_AND_DIFFUSE);

	//Materials
	glMaterialfv(GL_FRONT,GL_SPECULAR,MATERIAL);
	glMaterialf(GL_FRONT,GL_SHININESS,32.0f);

	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);

	//**
	// Texturecoords generation parameters
	//**

	InitGLTexGen();

	//Disable at the beginning
	//glDisable(GL_TEXTURE_2D);

	
	/************************
			OBJECTS
	*************************/

	objects.init();
	spot.init();

	//***** TIMER ******

	timer.create();
}

void CMainWindow::OnSysCommand(UINT nID, LPARAM lParam) 
{
	if (nID == SC_SCREENSAVE || 
		nID == SC_MONITORPOWER)
	{
		return;
	}

	CWnd::OnSysCommand(nID, lParam);
}

void CMainWindow::CreateGLWindow()
{
	if (fullscreen)
	{
		screenMode.setDisplayMode(640,480,16);
	}
		
	
	//Windowcreation
	CString className = AfxRegisterWndClass(
		CS_HREDRAW | CS_VREDRAW | CS_OWNDC,
		NULL,
		(HBRUSH)GetStockObject(BLACK_BRUSH),
		AfxGetApp()->LoadIcon(IDR_MAINFRAME));

	CreateEx(
		0,
		className,
		"Stone's Project Demo <www.steinsoft.net>",
		(fullscreen ? WS_POPUP : WS_POPUP|WS_CAPTION),
		(fullscreen ? CRect(0,0,640,480) : CRect(50,50,640,480)),
		NULL,
		0);

	//Show Window and make active
	ShowWindow(SW_SHOW);
	SetForegroundWindow();
	SetFocus();
		
	
	//No cursor visible
	ShowCursor(false);
}

int CMainWindow::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	//OpenGL init
	
	clientDC = new CClientDC(this);
	openGLDevice.create(clientDC->m_hDC);
	InitGL();
	
	return 0;
}

void CMainWindow::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	switch (nChar)
	{
	case VK_F9:
		fullscreen = !fullscreen;
		UpdateView();
		break;
	case VK_F7:
		Minimize();
		break;
	case VK_ADD:
		spot.toggleAnimation();
		break;
	case VK_SUBTRACT:
		spot.toggleAnimation();
		break;
	case VK_SPACE:
		spot.switchTexture();
		break;
	case VK_CONTROL:
		spot.dumpInfo();
		break;
	}
	
	CWnd::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CMainWindow::UpdateView()
{
	if (fullscreen)
	{
		screenMode.setDisplayMode(640,480,16);
		
		ModifyStyle(WS_CAPTION,0);
		MoveWindow(CRect(0,0,640,480));
	}
	else
	{	
		screenMode.reset();
		
		ModifyStyle(0,WS_CAPTION);
		MoveWindow(CRect(50,50,640,480));
	}

		
}

void CMainWindow::OnSetFocus(CWnd* pOldWnd) 
{
	CWnd::OnSetFocus(pOldWnd);
	
	UpdateView();
}

void CMainWindow::OnKillFocus(CWnd* pNewWnd) 
{
	CWnd::OnKillFocus(pNewWnd);
	
	Minimize();
	
}

void CMainWindow::Minimize()
{
	if (IsIconic())
	{
		return;
	}
		
	WINDOWPLACEMENT placement;
	GetWindowPlacement(&placement);

	if (fullscreen)
	{
		screenMode.reset();
	}

	placement.showCmd = SW_SHOWMINIMIZED;
	
	SetWindowPlacement(&placement);
}

void CMainWindow::InitGLTexGen()
{
	//coefficients for tex gen
	//(as a matrix, these 4 vec would be an identity matrix)
	float v1[] = {1.0f,0.0f,0.0f,0.0f};
	float v2[] = {0.0f,1.0f,0.0f,0.0f};
	float v3[] = {0.0f,0.0f,1.0f,0.0f};
	float v4[] = {0.0f,0.0f,0.0f,1.0f};

	//setup coefficients for tex gen
	glTexGenfv(GL_S,GL_EYE_PLANE,v1);
	glTexGenfv(GL_T,GL_EYE_PLANE,v2);
	glTexGenfv(GL_R,GL_EYE_PLANE,v3);
	glTexGenfv(GL_Q,GL_EYE_PLANE,v4);
	
	//setup tex gen mode
	//it will be eye linear
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
	glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
	glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);

	//Enable tex. coord generation
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	glEnable(GL_TEXTURE_GEN_R);
	glEnable(GL_TEXTURE_GEN_Q);
}
