// SSpot.cpp: Implementierung der Klasse SSpot.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SSpot.h"

#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define ROT_SPEED 0.05f

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

SSpot::SSpot()
{
	currentTexture = 0;
	position[0] = 2.0f;
	position[1] = 2.0f;
	rotation	= 0.0f;
	animation	= false;
}

SSpot::~SSpot()
{

}

void SSpot::loadTextures()
{
	textures[0].createMipmap("Data/smiley.bmp",GL_LINEAR_MIPMAP_NEAREST,
		GL_LINEAR,true);
	textures[1].createMipmap("Data/lightMap.bmp",GL_LINEAR_MIPMAP_NEAREST,
		GL_LINEAR,true);
	textures[2].createMipmap("Data/text.bmp",GL_LINEAR_MIPMAP_NEAREST,
		GL_LINEAR,true);
}


void SSpot::init()
{
	loadTextures();
}

void SSpot::beginDraw(float frametime)
{
	//*********************************************
	// UPDATE ROTATION (if animation is turned on)

	if (animation)
	{
		rotation += ROT_SPEED*frametime;
	}

	//*********************************************

	//Update Texture Matrix accord.
	//to the spot position
	glMatrixMode(GL_TEXTURE);
		glLoadIdentity();
		
		glRotatef(rotation,0.0f,0.0f,1.0f);
		glTranslatef(-position[0]+2.0f,-position[1]+2.0f,0.0f);
		
		//The lightmap will be bigger
		// here: scalefactor = bigger => image = smaller
		if (currentTexture != 1)
			glScalef(2.0f,2.0f,1.0f);
		
		gluPerspective(45.0f, 1, .1, 1);
	
		//Apply transform for our spot
		glTranslatef(position[0],position[1],0.0f);
	glMatrixMode(GL_MODELVIEW);
	
	glEnable(GL_TEXTURE_2D);
	textures[currentTexture].select();
	
	//The Spot should be transparent
	glEnable(GL_BLEND);
}

void SSpot::endDraw()
{
	//Disable/Enabled everyting that has been turned
	//off/on
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
}

void SSpot::move(float x, float y)
{
	position[0] += x;
	position[1] += y;
}

//DEL void SSpot::increaseSize()
//DEL {
//DEL 	size -= 0.25f;
//DEL }

//DEL void SSpot::decreaseSize()
//DEL {
//DEL 	size += 0.25f;
//DEL }

void SSpot::switchTexture()
{
	currentTexture += 1;

	if (currentTexture > 2)
	{
		currentTexture = 0;
	}
}

void SSpot::dumpInfo()
{
	TRACE3("Dump:\n Position %f,%f\n Size: %f\n",position[0],
					position[1],0);
}

void SSpot::toggleAnimation()
{	
	animation = !animation;
}
